
/*

     Copy the input stream to the argument file, overwriting rather
     than truncating that file if it already exists.

     The number of bytes to be written can be optionally specified.
     If the input stream doesn't contain precisely this number
     an error is reported.

*/

#include <stdio.h>
#include <fcntl.h>

#define BUFFER_SIZE 65536

static char *usage        = "USAGE: overwrite <file> [<n_bytes>]\n" ;

extern int  errno, sys_nerr ;
extern char *sys_errlist [] ;

void sys_assert (condition, usr_message)
/*

  Test `condition'.  Abort program, reporting `usr_message', if it fails.

*/
int condition ;
char *usr_message ;
{
  if (!condition)
  {
    fprintf (stderr, "ERROR: %s", usr_message) ;
    if ((errno > 0) && (errno < sys_nerr))
      fprintf (stderr, "... %s", sys_errlist [errno]) ;
    fprintf (stderr, ".\n") ;
    exit (1) ;
  }
}

void get_params (file, nbytes, argc, argv)
char **file ;
int *nbytes ;
int argc ;
char *argv [] ;
{
  if ((argc != 2) && (argc != 3))
  {
    fprintf (stderr, usage) ;
    exit (1) ;
  }
  *file = argv [1] ;
  if (argc == 3)
  {
    int result ;

    result = sscanf (argv [2], "%d", nbytes) ;
    if (result != 1)
    {
      fprintf (stderr, usage) ;
      exit (1) ;
    }
  }
  else
    *nbytes = -1 ;
}

void main (argc, argv)
int argc ;
char *argv [] ;
{
  char *file ;
  int desc ;
  int nbytes, count = 0 ;

  get_params (&file, &nbytes, argc, argv) ;
  desc = open (file, O_CREAT | O_WRONLY, 0x1a4) ;
  sys_assert (desc >= 0, "Failed to open file") ;

  while (1)
  {
    int ptr, residue ;
    char buf [BUFFER_SIZE] ;
    int result ;

    ptr     = 0 ;
    residue = BUFFER_SIZE ;
    while (residue > 0)
    {
      result   = read (0, buf + ptr, residue) ;
      if (result == 0)
      {
	result = write (desc, buf, ptr) ;
	sys_assert (result == ptr, "Write failed") ;
	count += ptr ;
	if ((nbytes >= 0) && (count != nbytes))
	{
	  fprintf (stderr, "ERROR: expecting %d bytes. Received %d.\n",
			    nbytes, count) ;
	  exit (1) ;
	}
	exit (0) ;
      }
      sys_assert (result > 0, "Read failed") ;
      ptr     += result ;
      residue -= result ;
    }
    result = write (desc, buf, BUFFER_SIZE) ;
    sys_assert (result == ptr, "Write failed") ;
    count += BUFFER_SIZE ;
  }
}

